@@ -71,28 +71,19 @@ pub enum Views {
 
 pub fn fetch_problem(problem_name: &str, problem_dir: &Path) -> MapmResult<Problem> {
     match fs::read_to_string(problem_dir.join(&[problem_name, ".yml"].concat())) {
-        Ok(problem_yaml) => match parse_problem_yaml(&problem_yaml) {
-            Success(problem) => {
-                return Success(problem);
-            }
-            err => {
-                return err;
-            }
-        },
-        Err(_) => {
-            return Fail(ProblemErr(
-                [
-                    "Could not read problem ",
-                    &problem_name,
-                    " from `",
-                    &problem_dir.to_str().unwrap(),
-                    "/",
-                    &problem_name,
-                    ".yml`",
-                ]
-                .concat(),
-            ));
-        }
+        Ok(problem_yaml) => parse_problem_yaml(&problem_yaml),
+        Err(_) => Fail(ProblemErr(
+            [
+                "Could not read problem ",
+                &problem_name,
+                " from `",
+                &problem_dir.to_str().unwrap(),
+                "/",
+                &problem_name,
+                ".yml`",
+            ]
+            .concat(),
+        )),
     }
 }
 
@@ -138,62 +129,42 @@ impl Problem {
         match filter {
             Exists { key } => {
                 if key == "solutions" {
-                    return self.solutions.len() > 0;
+                    self.solutions.len() > 0
                 } else {
-                    return self.vars.contains_key(&key);
+                    self.vars.contains_key(&key)
                 }
             }
             Eq { key, val } => match self.vars.get(&key) {
-                Some(problem_value) => {
-                    return problem_value == &val;
-                }
-                None => {
-                    return false;
-                }
+                Some(problem_value) => problem_value == &val,
+                None => false,
             },
             Gt { key, val } => match self.vars.get(&key) {
                 Some(problem_value) => match problem_value.parse::<u32>() {
-                    Ok(problem_value_u32) => {
-                        return problem_value_u32 > val;
-                    }
-                    Err(_) => return false,
+                    Ok(problem_value_u32) => problem_value_u32 > val,
+                    Err(_) => false,
                 },
-                None => {
-                    return false;
-                }
+                None => false,
             },
             Lt { key, val } => match self.vars.get(&key) {
                 Some(problem_value) => match problem_value.parse::<u32>() {
-                    Ok(problem_value_u32) => {
-                        return problem_value_u32 < val;
-                    }
-                    Err(_) => return false,
+                    Ok(problem_value_u32) => problem_value_u32 < val,
+                    Err(_) => false,
                 },
-                None => {
-                    return false;
-                }
+                None => false,
             },
             Ge { key, val } => match self.vars.get(&key) {
                 Some(problem_value) => match problem_value.parse::<u32>() {
-                    Ok(problem_value_u32) => {
-                        return problem_value_u32 >= val;
-                    }
-                    Err(_) => return false,
+                    Ok(problem_value_u32) => problem_value_u32 >= val,
+                    Err(_) => false,
                 },
-                None => {
-                    return false;
-                }
+                None => false,
             },
             Le { key, val } => match self.vars.get(&key) {
                 Some(problem_value) => match problem_value.parse::<u32>() {
-                    Ok(problem_value_u32) => {
-                        return problem_value_u32 <= val;
-                    }
-                    Err(_) => return false,
+                    Ok(problem_value_u32) => problem_value_u32 <= val,
+                    Err(_) => false,
                 },
-                None => {
-                    return false;
-                }
+                None => false,
             },
         }
     }
@@ -217,7 +188,7 @@ impl Problem {
                 }
             }
         }
-        return Some(self);
+        Some(self)
     }
 
     /// Show only the keys passed in or everything but the keys passed in to a problem
@@ -265,9 +236,9 @@ impl Problem {
                     }
                 }
                 if tags.contains(&"solutions".to_string()) {
-                    return (vars, Some(self.solutions));
+                    (vars, Some(self.solutions))
                 } else {
-                    return (vars, None);
+                    (vars, None)
                 }
             }
             Hide(tags) => {
@@ -278,9 +249,9 @@ impl Problem {
                     }
                 }
                 if !tags.contains(&"solutions".to_string()) {
-                    return (vars, Some(self.solutions));
+                    (vars, Some(self.solutions))
                 } else {
-                    return (vars, None);
+                    (vars, None)
                 }
             }
         }
@@ -475,3 +446,49 @@ pub fn parse_problem_yaml(yaml: &str) -> MapmResult<Problem> {
         }
     }
 }
+
+/// Converts a vector of problems into TeX files and writes them in the current working directory
+///
+/// Internal function used for compiling vectors of problems and for compiling contests
+///
+/// Returns a header TeX string as well (for contest compilation if necessary)
+
+pub(crate) fn write_as_tex(problem_vec: Vec<Problem>) -> String {
+    let mut problem_number = 0;
+    let mut headers = String::new();
+
+    for problem in problem_vec {
+        problem_number += 1;
+        headers.push_str(
+            &[
+                "\\expandafter\\def\\csname mapm@solcount@",
+                &problem_number.to_string(),
+                "\\endcsname{",
+                &problem.solutions.len().to_string(),
+                "}",
+            ]
+            .concat(),
+        );
+        for (key, val) in &problem.vars {
+            let filename = &["mapm-prob-", &problem_number.to_string(), "-", key, ".tex"].concat();
+            fs::write(filename, val).expect(&["Could not write to `", &filename, "`"].concat());
+        }
+        for (pos, map) in problem.solutions.iter().enumerate() {
+            for (key, val) in map {
+                let solution_number = &pos + 1;
+                let filename = &[
+                    "mapm-sol-",
+                    &problem_number.to_string(),
+                    "-",
+                    &solution_number.to_string(),
+                    "-",
+                    key,
+                    ".tex",
+                ]
+                .concat();
+                fs::write(filename, val).expect(&["Could not write to `", &filename, "`"].concat());
+            }
+        }
+    }
+    headers
+}
